import 'dart:developer';
import 'dart:io';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:nui_firebase/nui_firebase.dart';
import 'package:http/http.dart' as http;

class NUIFireStorage implements NUIFireComponent{
  NUIFireStorage._builder(NUIFireStorageBuilder builder){
    _bucket = builder._bucket;
    _storage = builder._storage;
  }

  String _bucket;
  FirebaseStorage _storage;
  static NUIFireStorage _instance;

  static NUIFireStorage get(){
    if(_instance == null) throw Exception("NUIFireStorage -> You have not initialized NUIFireStorage with builder");
    return _instance;
  }

  Future<FireResult<StorageTaskSnapshot>> uploadToPath(File file, String path, {StorageMetadata metadata}) async{
    return upload(file, _storage.ref().child(path), metadata: metadata);
  }

  Future<FireResult<StorageTaskSnapshot>> upload(File file, StorageReference storageReference, {StorageMetadata metadata}) async{
    StorageTaskSnapshot snapshot;
    try {
      final uploadTask = storageReference.putFile(file, metadata);
      snapshot = await uploadTask.onComplete;
    }catch(e){
      log("NUIFireStorage -> Failed to upload file");
      snapshot = null;
    }

    if(snapshot != null){
      return FireResult(success: true, message: "File uploaded successfully", data: snapshot);
    }
    else{
      return FireResult(success: false, message: "Failed to upload file", data: snapshot);
    }
  }

  Future<FireResult<File>> download(StorageReference storageReference, String filename) async{
    File finalFile;
    try {
      final String url = await storageReference.getDownloadURL();
      final http.Response downloadData = await http.get(url);
      final Directory systemTempDir = Directory.systemTemp;
      File tempFile = File('${systemTempDir.path}/$filename');
      if (tempFile.existsSync()) {
        await tempFile.delete();
      }
      await tempFile.create();
      assert(await tempFile.readAsString() == "");
      final StorageFileDownloadTask task = storageReference.writeToFile(tempFile);
      final int byteCount = (await task.future).totalByteCount;
      final String tempFileContents = await tempFile.readAsString();

      final String fileContents = downloadData.body;
      final String name = await storageReference.getName();
      final String bucket = await storageReference.getBucket();
      final String path = await storageReference.getPath();

      finalFile = tempFile;
    }
    catch(e){
      log("NUIFireStorage -> Failed to download file to $filename");
      finalFile = null;
    }

    if(finalFile != null){
      return FireResult(success: true, message: "File downloaded successfully to ${finalFile.path}", data: finalFile);
    }
    else{
      return FireResult(success: false, message: "Failed to download file");
    }
  }

  @override
  String name() => "NUIFireStorage";
}

class NUIFireStorageBuilder implements NUIFireBuilder<NUIFireStorage>{
  String _bucket;
  String _androidAppId;
  String _iosAppId;
  FirebaseStorage _storage;
  NUIFireStorageBuilder bucket(String bucket){
    _bucket = bucket;
  }

  NUIFireStorageBuilder androidAppId(String androidAppId){
    _androidAppId = androidAppId;
  }

  NUIFireStorageBuilder iosAppId(String iosAppId){
    _iosAppId = iosAppId;
  }

  @override
  NUIFireStorage build(FirebaseApp app){
    final FirebaseStorage storage = FirebaseStorage(app: app, storageBucket: _bucket);
    _storage = storage;
    return NUIFireStorage._builder(this);
  }
}